package com.vp.plugin.sample.constructorsupport.actions;

import java.awt.event.*;

import javax.swing.*;

import com.vp.plugin.*;
import com.vp.plugin.action.*;
import com.vp.plugin.model.*;
import com.vp.plugin.model.factory.*;

public abstract class AbstractAddConstructorActionController implements VPContextActionController {
	
	private String _lastVisibility = null;
	private int _methodKind;
	
	public AbstractAddConstructorActionController(int aMethodKind) {
		this._methodKind = aMethodKind;
	}
	
	public final void performAction(VPAction aAction, VPContext aContext, ActionEvent aE) {
		IClass lClass = (IClass) aContext.getModelElement();
		
		// create operation
		IOperation lConstructor = lClass.createOperation();
		lConstructor.setName(lClass.getName());
		if (_lastVisibility != null) {
			lConstructor.setVisibility(_lastVisibility);
		}
		
		IDotNetOperationCodeDetail lDetail = IModelElementFactory.instance().createDotNetOperationCodeDetail();
		lDetail.setMethodKind(_methodKind);
		lConstructor.setDotnetCodeDetail(lDetail);
		
		ApplicationManager.instance().getViewManager().openSpec(lConstructor, null);
		
		// verify not duplicated operation, if duplicated, delete the new constructor.
		IOperation[] lOperations = lClass.toOperationArray();
		int lOperationCount = lOperations == null ? 0 : lOperations.length;
		for (int lOperationIndex = 0; lOperationIndex < lOperationCount; lOperationIndex++) {
			IOperation lOperation = lOperations[lOperationIndex];
			if (! lConstructor.equals(lOperation)) {
				if (isOperationEquals(lConstructor, lOperation)) {
					IParameter[] lParameters1 = lConstructor.toParameterArray();
					IParameter[] lParameters2 = lOperation.toParameterArray();
					
					int lCount1 = lParameters1 == null ? 0 : lParameters1.length;
					int lCount2 = lParameters2 == null ? 0 : lParameters2.length;
					if (lCount1 == lCount2) {
						boolean lParametersMatch = true;
						for (int i = 0; i < lCount1 && lParametersMatch; i++) {
							IParameter lParameter1 = lParameters1[i];
							IParameter lParameter2 = lParameters2[i];
							
							if (! objectEquals(lParameter1.getType(), lParameter2.getType())) {
								lParametersMatch = false;
							}
						}
						if (lParametersMatch && false) {
							// delete the lConstructor
							ApplicationManager.instance().getViewManager().showMessageDialog(
									null, 
									"New constructor is duplicated with existing one, the new constructor will be removed.", 
									"Warning", 
									JOptionPane.WARNING_MESSAGE
							);
							lConstructor.delete();
							return;
						}
					}
				}
			}
		}
		
		_lastVisibility = lConstructor.getVisibility();
	}
	
	private boolean isOperationEquals(IOperation aOperation1, IOperation aOperation2) {
		// check constructor / destructor
		int lMethodKind1 = isOperationConstructorOrDestructor(aOperation1);
		int lMethodKind2 = isOperationConstructorOrDestructor(aOperation2);

		// don't check duplicate, seems same signature on constructor/destructor is allowed.
		if (lMethodKind1 == lMethodKind2 && lMethodKind1 == IDotNetOperationCodeDetail.METHOD_KIND_FUNCTION) {
			// check name
			return aOperation1.getName().equals(aOperation2.getName());
		}
		else {
			return false;
		}
	}
	// if Operation is not Constructor or Destructor, Function will be returned.
	private int isOperationConstructorOrDestructor(IOperation aOperation) {
		IDotNetOperationCodeDetail lDetail = aOperation.getDotnetCodeDetail();
		if (lDetail != null) {
			if (lDetail.getMethodKind() == IDotNetOperationCodeDetail.METHOD_KIND_CONSTRUCTOR) {
				return IDotNetOperationCodeDetail.METHOD_KIND_CONSTRUCTOR;
			}
			else if (lDetail.getMethodKind() == IDotNetOperationCodeDetail.METHOD_KIND_DESTRUCTOR) {
				return IDotNetOperationCodeDetail.METHOD_KIND_DESTRUCTOR;
			}
		}
		return IDotNetOperationCodeDetail.METHOD_KIND_FUNCTION;
	}
	
	private boolean objectEquals(Object aObj1, Object aObj2) {
		if (aObj1 == null) {
			return aObj2 == null;
		}
		else if (aObj2 == null) {
			return false;
		}
		else {
			return aObj1.equals(aObj2);
		}
	}
	
	public final void update(VPAction aAction, VPContext aContext) {
		IModelElement lModelElement = aContext.getModelElement();
		if (lModelElement instanceof IClass) {
			String lName = lModelElement.getNickname();
			if (lName != null && lName.length() > 0) {
				aAction.setEnabled(true);
				return;
			}
		}
		
		aAction.setEnabled(false);
	}
}